'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.VERSION = undefined;
exports.target = target;
exports.transform = transform;

var _package = require('../package.json');

Object.defineProperty(exports, 'VERSION', {
	enumerable: true,
	get: function get() {
		return _package.version;
	}
});

var _acorn = require('acorn');

var acorn = _interopRequireWildcard(_acorn);

var _inject = require('acorn-jsx/inject');

var _inject2 = _interopRequireDefault(_inject);

var _inject3 = require('acorn-dynamic-import/lib/inject');

var _inject4 = _interopRequireDefault(_inject3);

var _Program = require('./program/Program.js');

var _Program2 = _interopRequireDefault(_Program);

var _support = require('./support.js');

var _getSnippet = require('./utils/getSnippet.js');

var _getSnippet2 = _interopRequireDefault(_getSnippet);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var _reduce = [_inject2.default, _inject4.default].reduce(function (final, plugin) {
	return plugin(final);
}, acorn),
    parse = _reduce.parse;

var dangerousTransforms = ['dangerousTaggedTemplateString', 'dangerousForOf'];

function target(target) {
	var targets = Object.keys(target);
	var bitmask = targets.length ? 1048575 : 262144;

	Object.keys(target).forEach(function (environment) {
		var versions = _support.matrix[environment];
		if (!versions) throw new Error('Unknown environment \'' + environment + '\'. Please raise an issue at https://github.com/Rich-Harris/buble/issues');

		var targetVersion = target[environment];
		if (!(targetVersion in versions)) throw new Error('Support data exists for the following versions of ' + environment + ': ' + Object.keys(versions).join(', ') + '. Please raise an issue at https://github.com/Rich-Harris/buble/issues');
		var support = versions[targetVersion];

		bitmask &= support;
	});

	var transforms = Object.create(null);
	_support.features.forEach(function (name, i) {
		transforms[name] = !(bitmask & 1 << i);
	});

	dangerousTransforms.forEach(function (name) {
		transforms[name] = false;
	});

	return transforms;
}

function transform(source) {
	var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

	var ast = void 0;
	var jsx = null;

	try {
		ast = parse(source, {
			ecmaVersion: 9,
			preserveParens: true,
			sourceType: 'module',
			onComment: function onComment(block, text) {
				if (!jsx) {
					var match = /@jsx\s+([^\s]+)/.exec(text);
					if (match) jsx = match[1];
				}
			},
			plugins: {
				jsx: true,
				dynamicImport: true
			}
		});
		options.jsx = jsx || options.jsx;
	} catch (err) {
		err.snippet = (0, _getSnippet2.default)(source, err.loc);
		err.toString = function () {
			return err.name + ': ' + err.message + '\n' + err.snippet;
		};
		throw err;
	}

	var transforms = target(options.target || {});
	Object.keys(options.transforms || {}).forEach(function (name) {
		if (name === 'modules') {
			if (!('moduleImport' in options.transforms)) transforms.moduleImport = options.transforms.modules;
			if (!('moduleExport' in options.transforms)) transforms.moduleExport = options.transforms.modules;
			return;
		}

		if (!(name in transforms)) throw new Error('Unknown transform \'' + name + '\'');
		transforms[name] = options.transforms[name];
	});

	return new _Program2.default(source, ast, transforms, options).export(options);
}