'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.default = Scope;

var _extractNames = require('./extractNames.js');

var _extractNames2 = _interopRequireDefault(_extractNames);

var _reserved = require('../utils/reserved.js');

var _reserved2 = _interopRequireDefault(_reserved);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function Scope(options) {
	options = options || {};

	this.parent = options.parent;
	this.isBlockScope = !!options.block;
	this.createDeclarationCallback = options.declare;

	var scope = this;
	while (scope.isBlockScope) {
		scope = scope.parent;
	}this.functionScope = scope;

	this.identifiers = [];
	this.declarations = Object.create(null);
	this.references = Object.create(null);
	this.blockScopedDeclarations = this.isBlockScope ? null : Object.create(null);
	this.aliases = Object.create(null);
}

Scope.prototype = {
	addDeclaration: function addDeclaration(node, kind) {
		var _iteratorNormalCompletion = true;
		var _didIteratorError = false;
		var _iteratorError = undefined;

		try {
			for (var _iterator = (0, _extractNames2.default)(node)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
				var identifier = _step.value;

				var name = identifier.name;

				var declaration = { name: name, node: identifier, kind: kind, instances: [] };
				this.declarations[name] = declaration;

				if (this.isBlockScope) {
					if (!this.functionScope.blockScopedDeclarations[name]) this.functionScope.blockScopedDeclarations[name] = [];
					this.functionScope.blockScopedDeclarations[name].push(declaration);
				}
			}
		} catch (err) {
			_didIteratorError = true;
			_iteratorError = err;
		} finally {
			try {
				if (!_iteratorNormalCompletion && _iterator.return) {
					_iterator.return();
				}
			} finally {
				if (_didIteratorError) {
					throw _iteratorError;
				}
			}
		}
	},
	addReference: function addReference(identifier) {
		if (this.consolidated) {
			this.consolidateReference(identifier);
		} else {
			this.identifiers.push(identifier);
		}
	},
	consolidate: function consolidate() {
		for (var i = 0; i < this.identifiers.length; i += 1) {
			// we might push to the array during consolidation, so don't cache length
			var identifier = this.identifiers[i];
			this.consolidateReference(identifier);
		}

		this.consolidated = true; // TODO understand why this is necessary... seems bad
	},
	consolidateReference: function consolidateReference(identifier) {
		var declaration = this.declarations[identifier.name];
		if (declaration) {
			declaration.instances.push(identifier);
		} else {
			this.references[identifier.name] = true;
			if (this.parent) this.parent.addReference(identifier);
		}
	},
	contains: function contains(name) {
		return this.declarations[name] || (this.parent ? this.parent.contains(name) : false);
	},
	createIdentifier: function createIdentifier(base) {
		if (typeof base === 'number') base = base.toString();

		base = base.replace(/\s/g, '').replace(/\[([^\]]+)\]/g, '_$1').replace(/[^a-zA-Z0-9_$]/g, '_').replace(/_{2,}/, '_');

		var name = base;
		var counter = 1;

		while (this.declarations[name] || this.references[name] || this.aliases[name] || name in _reserved2.default) {
			name = base + '$' + counter++;
		}

		this.aliases[name] = true;
		return name;
	},
	createDeclaration: function createDeclaration(base) {
		var id = this.createIdentifier(base);
		this.createDeclarationCallback(id);
		return id;
	},
	findDeclaration: function findDeclaration(name) {
		return this.declarations[name] || this.parent && this.parent.findDeclaration(name);
	},


	// Sometimes, block scope declarations change name during transpilation
	resolveName: function resolveName(name) {
		var declaration = this.findDeclaration(name);
		return declaration ? declaration.name : name;
	}
};